@@ -2,6 +2,10 @@
 Provides functions to perform operations on ℝ³ Vectors centered at the
 Origin (0, 0, 0)"""
 
+from math import acos, cos, degrees, exp, hypot, isclose, radians, sin
+import matplotlib.pyplot as plt
+from random import uniform
+
 __name__ = "__main__"
 
 
@@ -50,8 +54,6 @@ class Vector():
             if (vec.count(1), vec.count(0)) == (1, 2):
                 raise AttributeError("cannot create Vectors 'î', 'ĵ', 'k̂'")
 
-        from math import hypot
-
         self.tuple = tuple(vec)
         self.x, self.y, self.z = self.i, self.j, self.k = self.tuple
         self.magnitude = hypot(*self.tuple)
@@ -189,6 +191,20 @@ class Vector():
             return Vector.components(self) == Vector.components(other)
 
 
+    def __lt__(self, other, /):
+        """compares magnitudes of Vector Objects using '<' operator
+        returns True if magnitude of self is less than magnitude of other"""
+
+        return Vector.mod(self) < Vector.mod(other)
+
+
+    def __gt__(self, other, /):
+        """compares magnitudes of Vector Objects using '>' operator
+        returns True if magnitude of self is greater than magnitude of other"""
+
+        return Vector.mod(self) > Vector.mod(other)
+
+
     def __add__(self, other, /):
         """add Vector Objects using '+' operator
         returns the result for (self + other)"""
@@ -263,12 +279,10 @@ class Vector():
                 "invalid operand for 'mod': must be positive 'int' / 'float'"
             )
 
-        from math import cos, radians
-
         cosines = [cos(radians(angle)) for angle in (alpha, beta, gamma)]
         total = sum(dc**2 for dc in cosines)
 
-        if round(total, 10) != 1:
+        if not isclose(total, 1):
             raise ValueError(" ".join((
                 "given angles do not form direction cosines:",
                 "see help(FromAngles) for help on valid angles"
@@ -339,9 +353,7 @@ class Vector():
         if not vectors:
             return
         else:
-            import matplotlib.pyplot as plt
-
-        Vector.ExceptionVector(*vectors)
+            Vector.ExceptionVector(*vectors)
 
         ax = plt.axes(projection="3d")
         colors = ["red", "orange", "green", "blue", "purple"] * len(vectors)
@@ -524,12 +536,20 @@ class Vector():
         axis_angles(FromAngles(𝜶, 𝜷, 𝜸, mod=n)) == (𝜶, 𝜷, 𝜸)
         see help(FromAngles) for help on angles"""
 
-        from math import acos, degrees
         return tuple(
             [round(degrees(acos(i)), 10) for i in Vector.directioncos(self)]
         )
 
 
+    def polar_repr(self, /):
+        """returns a str of representation of the Vector the Polar form
+        polar_repr(FromAngles(𝜶, 𝜷, 𝜸, mod=n)) returns a str of the form
+        n(cos(𝜶)î + cos(𝜷)ĵ + cos(𝜸)k̂)"""
+
+        alpha, beta, gamma = Vector.axis_angles(self)
+        return f"{self.mod()}(cos({alpha})î + cos({beta})ĵ + cos({gamma})k̂"
+
+
     def dot(self, other, /):
         """performs dot product / scalar product on two Vectors and
         returns the result (scalar)
@@ -594,7 +614,6 @@ class Vector():
         Note: using Vector.null as either of the operands raises Exception"""
 
         Vector.ExceptionNull(self, other)
-        from math import acos, degrees
         result = self.dot(other) / (self.magnitude * other.magnitude)
         return round(degrees(acos(round(result, 10))), 10)
 
@@ -667,7 +686,7 @@ class Vector():
     def isunit(self, /):
         """returns True if the given Vector is a unit Vector, False otherwise"""
 
-        return Vector.mod(self) == 1
+        return isclose(Vector.mod(self), 1)
 
 
     def isinverse(self, other, /):
@@ -723,9 +742,7 @@ class Vector():
         if not mod:
             return Vector.null
 
-        from random import uniform
         result = uniform(-1, 1), uniform(-1, 1), uniform(-1, 1)
-
         return Vector.FromSequence(result).unit().scale(mod)
 
 
@@ -788,6 +805,89 @@ class Vector():
             return 1
 
 
+    def rotate(self, /, *, yaw, pitch, roll):
+        """rotates the Vector by the given yaw, pitch and roll angles using
+        the Right-Hand Rule (angles are in degrees taken counter-clockwise)
+        formally, this corresponds to an intrinsic rotation whose Tait-Bryan
+        angles are yaw, pitch and roll about the Z, Y and X axes respectively"""
+
+        x, y, z = Vector.components(self)
+
+        for angle in yaw, pitch, roll:
+            if not isinstance(angle, (int, float)):
+                raise TypeError(" ".join((
+                    "inalid operand type(s) for 'yaw', 'pitch' or 'roll':",
+                    "must be 'int' / 'float'"
+                )))
+
+        cA, sA = cos(radians(yaw)), sin(radians(yaw))
+        cB, sB = cos(radians(pitch)), sin(radians(pitch))
+        cC, sC = cos(radians(roll)), sin(radians(roll))
+
+        X = x*cA*cB + y*(cA*sB*sC - sA*cC) + z*(cA*sB*cC + sA*sC)
+        Y = x*sA*cB + y*(sA*sB*sC + cA*cC) + z*(sA*sB*cC - cA*sC)
+        Z = -x*sB + y*cB*sC + z*cB*cC
+
+        return round(Vector.FromSequence([X, Y, Z]), 10)
+
+
+    def rotate_vector(self, other, /, *, phi=90):
+        """rotates the Vector self phi degrees about the Vector other using
+        the Right-Hand Rule. phi can be any Real Number.
+        when phi > 0, the rotation will be counter-clockwise when 'other' points
+        towards the observer.
+        a Vector rotated about its own axis returns itself
+        Note: other cannot be Vector.null"""
+
+        Vector.ExceptionVector(self, other)
+
+        if not isinstance(phi, (int, float)):
+            raise TypeError(
+                "invalid operand type 'phi': must be 'int' / 'float'"
+            )
+        phi = radians(phi)
+        other = other.unit()
+        result1 = other.scale(self.dot(other))
+        result2 = other.vector_triple(other, self).scale(-cos(phi))
+        result3 = other.cross(self).scale(sin(phi))
+
+        return round(sum([result1, result2, result3], Vector.null), 10)
+
+
+    def rotate_axis(self, /, axis, *, phi=90):
+        """rotates the Vector by phi degrees about positive direction of the
+        given axis using the Right-Hand Rule. phi can be any Real Number.
+        when phi > 0, the rotation will be counter-clockwise when the positive
+        direction of axis of rotation points towards the observer.
+        a Vector rotated about its own axis returns itself
+        axes defined here:
+          'x' == 'i' = X-axis
+          'y' == 'j' = Y-axis
+          'z' == 'k' = Z-axis"""
+
+        Vector.ExceptionVector(self)
+
+        if not isinstance(axis, str):
+            raise TypeError("invalid operand type for 'axis': must be 'str'")
+
+        if axis not in {"x", "y", "z", "i", "j", "k"}:
+            raise ValueError(" ".join((
+                "invalid operand for 'axis':",
+                "see help(rotate_axis) for help on valid axes"
+            )))
+        return self.rotate_vector(
+            (Vector.i if axis in {"x", "i"} else Vector.j if axis in {"y", "j"}
+            else Vector.k), phi=phi
+        )
+
+
+    def exp(self, /):
+        """returns a Vector in the direction of self whose magnitude is equal to
+        the exponential of the magnitude of self"""
+
+        return Vector.unit(self).scale(exp(Vector.mod(self)))
+
+
 Vector.i = Vector(1, 0, 0)
 Vector.j = Vector(0, 1, 0)
 Vector.k = Vector(0, 0, 1)
